%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Jennifer Jackson 2017                                                        %
% A Biodiversity Approach to Cyber Security                                    %
% PhD Complexity Science                                                       %
% University of Warwick                                                        %
% http://wrap.warwick.ac.uk/98458                                              %
% This software is distributed under the  Creative Commons Attribution License %
% https://creativecommons.org/licenses/by/4.0/                                 %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function rwpInit(rwph)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Static checks
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if rwph.tinrv < rwph.tinrg
    
    fprintf('Total (absolute) virus TX time is sooner than ');
    fprintf('total (absolute) genotype TX time\n');
    return;
    
end;

if rwph.xminln < rwph.xmin || rwph.xmaxln > rwph.xmax || ...
   rwph.yminln < rwph.ymin || rwph.ymaxln > rwph.ymax

    fprintf('Limited node region outside main node region');
    return;
    
end;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Setup node arrays (1R x 'nodes'C)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Set timestep to 0
rwph.t = 0;

% Setup limited and full range node positions
rwph.nodesfr = rwph.nodes - rwph.nodesln;
if rwph.nodesln > 0
    rwph.nodeslnminind = 1;
    rwph.nodeslnmaxind = 1 + rwph.nodesln - 1;
    rwph.nodesfrminind = rwph.nodeslnmaxind + 1;
    rwph.nodesfrmaxind = rwph.nodes;
else
    rwph.nodeslnminind = 0;
    rwph.nodeslnmaxind = 0;
    rwph.nodesfrminind = 1;
    rwph.nodesfrmaxind = rwph.nodes;
end;
lnmin = rwph.nodeslnminind;
lnmax = rwph.nodeslnmaxind;
frmin = rwph.nodesfrminind;
frmax = rwph.nodesfrmaxind;

% Generate min/max arrays for nodes
rwph.xminv = zeros(1, rwph.nodes);
rwph.xmaxv = zeros(1, rwph.nodes);
rwph.yminv = zeros(1, rwph.nodes);
rwph.ymaxv = zeros(1, rwph.nodes);
if rwph.nodesln > 0
rwph.xminv(lnmin:lnmax) = rwph.xminln .* ones(1, rwph.nodesln);
rwph.xmaxv(lnmin:lnmax) = rwph.xmaxln .* ones(1, rwph.nodesln);
rwph.yminv(lnmin:lnmax) = rwph.yminln .* ones(1, rwph.nodesln);
rwph.ymaxv(lnmin:lnmax) = rwph.ymaxln .* ones(1, rwph.nodesln);
end;
rwph.xminv(frmin:frmax) = rwph.xmin   .* ones(1, rwph.nodesfr);
rwph.xmaxv(frmin:frmax) = rwph.xmax   .* ones(1, rwph.nodesfr);
rwph.yminv(frmin:frmax) = rwph.ymin   .* ones(1, rwph.nodesfr);
rwph.ymaxv(frmin:frmax) = rwph.ymax   .* ones(1, rwph.nodesfr);

% Generate uniform random starting positions
rwph.cx = rwph.xminv + (rwph.xmaxv - rwph.xminv) .* rand(1, rwph.nodes);
rwph.cy = rwph.yminv + (rwph.ymaxv - rwph.yminv) .* rand(1, rwph.nodes);

% Initially set the last coordinates to the current (initial) - for plots
rwph.lx = rwph.cx;
rwph.ly = rwph.cy;

% Initially set the target to minx, miny
rwph.tx = rwph.xminv .* ones(1, rwph.nodes);
rwph.ty = rwph.yminv .* ones(1, rwph.nodes);

% Initially set the delta to zero
rwph.dx = zeros(1, rwph.nodes);
rwph.dy = zeros(1, rwph.nodes);

% Initially set no speed
rwph.sseg = zeros(1, rwph.nodes);

% Initially set no angle
rwph.aseg = zeros(1, rwph.nodes);

% Initially set no distance
rwph.dseg = zeros(1, rwph.nodes);

% Set segment time to -pmax indicate no target
rwph.tseg = (-rwph.pmax) .* ones(1, rwph.nodes);

% Set node pause time for current segment to -pausemax initially (no target)
rwph.pseg = rwph.pmax .* ones(1, rwph.nodes);

% Setup delta distance matrix ('nodes'R x 'nodes'C)
rwph.ddx = zeros(rwph.nodes, rwph.nodes);
rwph.ddy = zeros(rwph.nodes, rwph.nodes);
rwph.dd  = zeros(rwph.nodes, rwph.nodes);

% Setup timesteps-in-range matrix ('nodes'R x 'nodes'C)
rwph.tir = zeros(rwph.nodes, rwph.nodes);

% Setup tag matrix ('nodes'R x 'nodes'C)
rwph.tag = zeros(rwph.nodes, rwph.nodes);

% Setup a leading diagonal matrix with large 'intmax' on the diagonal
% - use this to prevent nodes seeing themselves as distance = 0
rwph.eyeo = eye(rwph.nodes) .* rwph.dexceed;

% Setup an NxN array that can be used to convert a single tag in a row
% into a node index number - each row of the matrix should hold a one-hot
% tag flag - so want to turn the column number into a node number, which
% represents the number of the node tagged by this node (row)
rwph.c2ind = repmat((1:1:rwph.nodes),rwph.nodes, 1);

% Setup the in-range list cell array (initially empty)
rwph.irl = cell(1, rwph.nodes);
rwph.irf = zeros(1, rwph.nodes);

% Setup the arrays for marking genotype and virus transmission
rwph.mlg = zeros(1, rwph.nodes);
rwph.mlv = zeros(1, rwph.nodes);

% Initially set based on starting locations
rwph.xlow  = min(rwph.cx);
rwph.xhigh = max(rwph.cx);
rwph.ylow  = min(rwph.cy);
rwph.yhigh = max(rwph.cy);
if rwph.nodesln > 0 
    rwph.xlowln  = min(rwph.cx(lnmin:lnmax));
    rwph.xhighln = max(rwph.cx(lnmin:lnmax));
    rwph.ylowln  = min(rwph.cy(lnmin:lnmax));
    rwph.yhighln = max(rwph.cy(lnmin:lnmax));
end;

% Make the logfile directory, if needed
if exist(rwph.logDir, 'file') ~= 7
    mkdir(rwph.logDir);
end;

% Setup file handles for logs
rwph.logTrack = zeros(1, rwph.nodes);
rwph.logDist  = zeros(1, rwph.nodes);
rwph.logTIR   = zeros(1, rwph.nodes);
if ~isempty(rwph.logena)
    for m = rwph.logena
        
        % Track log
        s = sprintf('%s\\rwpLogTrack-Node-%05i.txt', rwph.logDir, m);
        rwph.logTrack(m) = fopen(s, 'wt');
        fprintf(rwph.logTrack(m), '# Track Log - Node %05i\n', m);
        fprintf(rwph.logTrack(m), '#\n');
        fprintf(rwph.logTrack(m), '#   ');
        fprintf(rwph.logTrack(m), 'TS         ');
        fprintf(rwph.logTrack(m), 'LX         ');
        fprintf(rwph.logTrack(m), 'LY         ');
        fprintf(rwph.logTrack(m), 'CX         ');
        fprintf(rwph.logTrack(m), 'CY         ');
        fprintf(rwph.logTrack(m), 'TX         ');
        fprintf(rwph.logTrack(m), 'TY         ');
        fprintf(rwph.logTrack(m), 'DX         ');
        fprintf(rwph.logTrack(m), 'DY       ');
        fprintf(rwph.logTrack(m), 'SSEG       ');
        fprintf(rwph.logTrack(m), 'ASEG       ');
        fprintf(rwph.logTrack(m), 'DSEG   ');
        fprintf(rwph.logTrack(m), 'TSEG  ');
        fprintf(rwph.logTrack(m), 'PSEG    ');
        fprintf(rwph.logTrack(m), 'IRLV/IRL');
        fprintf(rwph.logTrack(m), '\n');
        
        % Delta Distance Log
        s = sprintf('%s\\rwpLogDist-Node-%05i.txt', rwph.logDir, m);
        rwph.logDist(m) = fopen(s, 'wt');
        fprintf(rwph.logDist(m), '# Distance Log - Node %05i\n', m);
        
        % Timesteps in range Log
        s = sprintf('%s\\rwpLogTIR-Node-%05i.txt', rwph.logDir, m);
        rwph.logTIR(m) = fopen(s, 'wt');
        fprintf(rwph.logTIR(m), '# Timesteps-in-range Log - Node %05i\n', m);
        
        % Genotype / Virus Flag Log
        s = sprintf('%s\\rwpLogGVFlag-Node-%05i.txt', rwph.logDir, m);
        rwph.logGVFlag(m) = fopen(s, 'wt');
        fprintf(rwph.logGVFlag(m), '# Genotype/Virus Transfer Flag Log - Node %05i\n', m);
        
     end;
end;

% Generate log entries as needed
rwpLog(rwph);
