%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Jennifer Jackson 2017                                                        %
% A Biodiversity Approach to Cyber Security                                    %
% PhD Complexity Science                                                       %
% University of Warwick                                                        %
% http://wrap.warwick.ac.uk/98458                                              %
% This software is distributed under the  Creative Commons Attribution License %
% https://creativecommons.org/licenses/by/4.0/                                 %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function rwpIterate(rwph)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Setup node arrays (1R x 'nodes'C)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Work out which need their dx,dy zeroing to effect a pause
nt = find(rwph.tseg == 0);
if ~isempty(nt)
    
    rwph.dx(nt) = 0.0;
    rwph.dy(nt) = 0.0;
    
end;


% Work out which need a new target (pause elapsed based on segment pause
% value per node)
nt = find(rwph.tseg <= (-rwph.pseg));
if ~isempty(nt)

    % Vector assign new speeds to all that need them
    rwph.sseg(nt) = rwph.smin + (rwph.smax - rwph.smin) .* rand(1, length(nt));

    % Vector assign an initial new target for all that need them
    rwph.tx(nt) = rwph.xminv(nt) + (rwph.xmaxv(nt) - rwph.xminv(nt)) .* rand(1, length(nt)); 
    rwph.ty(nt) = rwph.xminv(nt) + (rwph.xmaxv(nt) - rwph.xminv(nt)) .* rand(1, length(nt));

    % Calculate the initial distance for new targets for all that need them
    rwph.dseg(nt) = sqrt(((rwph.tx(nt) - rwph.cx(nt)) .^ 2) + ...
                         ((rwph.ty(nt) - rwph.cy(nt)) .^ 2));

    % Calculate the initial angle for new targets for all that need them
    rwph.aseg(nt) = atan2(rwph.ty(nt) - rwph.cy(nt), ...
                          rwph.tx(nt) - rwph.cx(nt));

    % Calculate the number of integer timesteps needed at this speed - always
    % cut the distance short to avoid going outside the boundaries
    rwph.tseg(nt) = floor(rwph.dseg(nt) ./ rwph.sseg(nt));

    % Vector assign new pause to all that need them
    rwph.pseg(nt) = randi([rwph.pmin rwph.pmax],1, length(nt));
    
    % Calculate the actual dx and dy per timestep
    rwph.dx(nt) = cos(rwph.aseg(nt)) .* rwph.sseg(nt);
    rwph.dy(nt) = sin(rwph.aseg(nt)) .* rwph.sseg(nt);

    % Calculate the actual target location based on dx,dy and timesteps
    rwph.tx(nt) = rwph.cx(nt) + (rwph.dx(nt) .* rwph.tseg(nt));
    rwph.ty(nt) = rwph.cy(nt) + (rwph.dy(nt) .* rwph.tseg(nt));

    % Calculate the actual distance for new targets for all that need them
    rwph.dseg(nt) = sqrt(((rwph.tx(nt) - rwph.cx(nt)) .^ 2) + ...
                         ((rwph.ty(nt) - rwph.cy(nt)) .^ 2));

    % Calculate the actual angle for new targets for all that need them
    rwph.aseg(nt) = atan2(rwph.ty(nt) - rwph.cy(nt), ...
                          rwph.tx(nt) - rwph.cx(nt));

end;

% Calculate the update - FOR ALL NODES
rwph.lx = rwph.cx;
rwph.ly = rwph.cy;
rwph.cx = rwph.cx + rwph.dx;
rwph.cy = rwph.cy + rwph.dy;

% Update delta distance matrix ('nodes'R x 'nodes'C)
%
%    Calculate the {x,y} differences by copying out the
%    current coordinate positions, one row for each node,
%    and then subtracting each node's {x,y} coordinate from 
%    all of the corresponding row entries to give their {x,y}
%    delta
rwph.ddx = repmat(rwph.cx, rwph.nodes, 1);
rwph.ddy = repmat(rwph.cy, rwph.nodes, 1);
for j = 1:rwph.nodes
    rwph.ddx(j,:) = rwph.ddx(j,:) - rwph.cx(j);
    rwph.ddy(j,:) = rwph.ddy(j,:) - rwph.cy(j);
end;
rwph.dd = sqrt((rwph.ddx .^ 2) + (rwph.ddy .^ 2));

% Add 'intmax' to all of the diagonals to prevent nodes
% seeing themselves as in range at 0 distance
rwph.dd = rwph.dd + rwph.eyeo;

% Iterate over nodes to find those in range for this timestep - the per
% row list extraction is fast, but likely difficult to try an do as a 
% whole matrix since each row is different
%
% Also process any time-in-range and tag marking updates here
%
% Processing of tag updates is done separately to ensure that nodes 
% are not tagged and released within the same cycle if 
% the genotype / virux tx time limit is zero
%
for j = 1:rwph.nodes
    
    % Use vector operations to find all values less than or equal to the
    % specified value - after thresholding, use a sort() to get all of the
    % relevant indicies, and then a find() to generate a reduced-legnth
    % array which contain only the indicies whose value were LTE the
    % value given
    %
    % Finally, use the list of indicies to look-up the values and then
    % sort the indicies to provide a return that is in ascending value order
    %
    % The matlab sort() is in ascending order (lowest first)
    %
    %flags = (rwph.dd(j,:) <= rwph.dinr);
    %[valid, list] = sort(flags);
    %sorted = valid .* list;
    %wanted = find(sorted);
    %indicies = sorted(wanted);
    indicies = find(rwph.dd(j,:) <= rwph.dinr);
    reordered = [rwph.dd(j,indicies); indicies];
    indiciesretvals = sortrows(reordered')';
    ir = indiciesretvals(2,:);

    % 'ir' is a cell array with the in range node indices ordered closest 
    % first (lowest index = closest)
    rwph.irl{j} = ir;
    if ~isempty(ir)
        rwph.irf(j) = (max(ir) >= 0);
    else
        rwph.irf(j) = 0;
    end;
    
    % For this node, clear all of the timesteps-in-range values to zero
    % in the temporary tiru array and next set values of 1.0 for all of 
    % the nodes in range on this timestep.
    %
    % Next treat any counts that have reached the virus tx time (or 
    % genotype tx time if the virus tx time is the same) since these
    % should be zero, and then increment in the next step if still in
    % range
    %
    % Next clear the count for nodes that have either txed or 
    % that are now out of range - this is done by multiplying by 0 or 1
    % by the tiru[] array calculated above, since this flags if a node
    % is in range or not by using 0.0 or 1.0 as values.  Then, add tiru[]
    % to increment the count of any node that is still in range, or 
    % increment it from 0.0 to 1.0 if it has come into range
    tiru = zeros(1, rwph.nodes);
    tiru(ir) = 1.0;
    nottxed = (rwph.tir(j,:) < rwph.tinrv);
    rwph.tir(j,:) = (rwph.tir(j,:) .* nottxed);
    rwph.tir(j,:) = (rwph.tir(j,:) .* tiru) + tiru;
    
    % Clear tags for any nodes that have gone out of range OR if they
    % have just txed a g or g+v, since these are treated as if gone 
    % out of range to bump the algorithm to reselect the tag according
    % to the mode chosen
    rwph.tag(j,:) = rwph.tag(j,:) .* tiru .* nottxed;
    
    % Tag new nodes as needed on first occurrence - this can only
    % occur if this node has another in range; this check alone
    % ensures that there is at least one potential node in range 
    % in the code that follows
    %
    % ALSO check here that the current node (row) does not have a tag
    % to ensure that each only has one at a time
    %
    if rwph.irf(j) > 0 && sum(rwph.tag(j, :)) == 0.0 
        
        % Copy the in-range list to make it easier to remove nodes that
        % fail the criteria checks
        irc = ir;
        
        % Determine what to do based on the mode
        switch rwph.mode
            
            % Random-in-range mode lets this node tag a random choice from
            % those in rage, even if they are already tagged
            case rwph.MODE_RIR
                
                % Don't care what's tagged, so no check - just select
                % randomly from those in range this cycle - also reset
                % the tir counter to 1, since want to restart when
                % newly tagged
                rv = randi(length(irc));
                rwph.tag(j, irc(rv)) = 1.0;
                rwph.tir(j, irc(rv)) = 1;
                
            % Nearest-in-range mode always chooses the closest, even
            % if already tagged
            case rwph.MODE_NIR
                
                % Don't care what's tagged, so no check - just select
                % the first index since this is the nearest as determined
                % above
                rwph.tag(j, irc(1)) = 1.0;
                rwph.tir(j, irc(1)) = 1;
            
            % Available-in-range mode is like Random-in-range, but
            % avoids any already tagged
            %
            % !!! - The ordering of who gets to pick first is done - !!!
            % !!! - purely in node order (1..nodes)                - !!!
            %
            case rwph.MODE_AIR
                
                % Zero out any nodes who are tagged by the target
                for k = 1:length(irc)
                    
                    % If the target node is tagged by anyone, the sum
                    % will be > 0, so remove it
                    if sum(rwph.tag(:,irc(k))) > 0
                        
                        % This ir entry is already in use, so zero it
                        irc(k) = 0.0;
                    end;
                end;
                
                % Update the copy to only include non-zero values - this
                % may make it undefined; this works by generating an array
                % of the indicies which are non-zero, and then getting
                % Matlab to matrix assign on these slice values
                irc = irc(find(irc));
                
                % The empty check ensures that there is no tagging, or
                % encounter if all the nodes in range are already
                % tagged
                if ~isempty(irc)
                    
                    % Can now just select randomly from those left
                    % in range this cycle
                    rv = randi(length(irc));
                    rwph.tag(j, irc(rv)) = 1.0;
                    rwph.tir(j, irc(rv)) = 1;
                    
                end;
        
            otherwise
                
                fprintf('Invalid enumeration value for mode\n');
                return; 
        
        end;
    
    end;
    
end;

% Process tags - determine those nodes that need a genotype or virus 
% transmission because they have been in range for >= the genotype or
% virus tx limit; also clear any nodes from the tag list once the
% genotype 
%
% Tag processing is done after tag clear / tag assignment to ensure
% that nodes are locked via the tag for at least one cycle, and that 
% they cannot be used and released within a single cycle if the genotype
% or virus tx time is set to zero

% Get an array of all of the tagged nodes whose time-in-range is 
% exactly the genotype tx time
ngt = ((rwph.tag .* rwph.tir) == rwph.tinrg);

% There should only be one entry per row, althogh the same number may
% occur multiple times in a column - otherwise a single node is taking
% genotype data from multiple sources...
if sum(ngt,2) > ones(rwph.nodes, 1)
    fprintf('One or more nodes has tagged multiple others for genotype tx\n');
    return;
end;

% Setup the return default as zero every timestep, and then use
% the masking array to convert the column index of the c2ind array
% to give the number of the node that is tagged by this node (row entry)
rwph.mlg = zeros(1, length(rwph.mlg)) + sum(rwph.c2ind .* ngt, 2)';

% Get an array of all of the tagged nodes whose time-in-range is 
% exactly the virus tx time
nvt = ((rwph.tag .* rwph.tir) == rwph.tinrv);

% There should only be one entry per row, althogh the same number may
% occur multiple times in a column - otherwise a single node is taking
% virus data from multiple sources...
if sum(nvt,2) > ones(rwph.nodes, 1)
    fprintf('One or more nodes has tagged multiple others for virus tx\n');
    return;
end;

% Setup the return default as zero every timestep, and then use
% the masking array to convert the column index of the c2ind array
% to give the number of the node that is tagged by this node (row entry)
rwph.mlv = zeros(1, length(rwph.mlv)) + sum(rwph.c2ind .* nvt, 2)';

% Track the most extreme waypoint locations
rwph.xlow    = min([rwph.xlow    rwph.cx]);
rwph.xhigh   = max([rwph.xhigh   rwph.cx]);
rwph.ylow    = min([rwph.ylow    rwph.cy]);
rwph.yhigh   = max([rwph.yhigh   rwph.cy]);
if rwph.nodesln > 0
rwph.xlowln  = min([rwph.xlowln  rwph.cx(rwph.nodeslnminind:rwph.nodeslnmaxind)]);
rwph.xhighln = max([rwph.xhighln rwph.cx(rwph.nodeslnminind:rwph.nodeslnmaxind)]);
rwph.ylowln  = min([rwph.ylowln  rwph.cy(rwph.nodeslnminind:rwph.nodeslnmaxind)]);
rwph.yhighln = max([rwph.yhighln rwph.cy(rwph.nodeslnminind:rwph.nodeslnmaxind)]);
end
% Decrement the segment counts
rwph.tseg = rwph.tseg - 1;

% Increment the timestep
rwph.t = rwph.t + 1;

% Generate log entries as needed
rwpLog(rwph);
